import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'intl/messages_all.dart';

class AppLocalizer {
  AppLocalizer();
  
  static late AppLocalizer current;
  
  static const AppLocalizationDelegate delegate =
    AppLocalizationDelegate();

  static Future<AppLocalizer> load(Locale locale) {
    final name = (locale.countryCode?.isEmpty ?? false) ? locale.languageCode : locale.toString();
    final localeName = Intl.canonicalizedLocale(name); 
    return initializeMessages(localeName).then((_) {
      Intl.defaultLocale = localeName;
      AppLocalizer.current = AppLocalizer();
      
      return AppLocalizer.current;
    });
  } 

  static AppLocalizer? of(BuildContext context) {
    return Localizations.of<AppLocalizer>(context, AppLocalizer);
  }

  /// `Please check your connection`
  String get pleaseCheckYourConnection {
    return Intl.message(
      'Please check your connection',
      name: 'please_check_your_connection',
      desc: '',
      args: [],
    );
  }

  /// `Server error`
  String get serverError {
    return Intl.message(
      'Server error',
      name: 'server_error',
      desc: '',
      args: [],
    );
  }

  /// `Access to location denied`
  String get accessToLocationDenied {
    return Intl.message(
      'Access to location denied',
      name: 'access_to_location_denied',
      desc: '',
      args: [],
    );
  }

  /// `Allow access to the location services.`
  String get allowAccessToTheLocationServices {
    return Intl.message(
      'Allow access to the location services.',
      name: 'allow_access_to_the_location_services',
      desc: '',
      args: [],
    );
  }

  /// `Access to location permanently denied`
  String get accessToLocationPermanentlyDenied {
    return Intl.message(
      'Access to location permanently denied',
      name: 'access_to_location_permanently_denied',
      desc: '',
      args: [],
    );
  }

  /// `Allow access to the location services for this App using the device settings.`
  String get allowAccessToTheLocationServicesFromSettings {
    return Intl.message(
      'Allow access to the location services for this App using the device settings.',
      name: 'allow_access_to_the_location_services_from_settings',
      desc: '',
      args: [],
    );
  }

  /// `Ok`
  String get ok {
    return Intl.message(
      'Ok',
      name: 'ok',
      desc: '',
      args: [],
    );
  }

  /// `Can't get current location`
  String get cantGetCurrentLocation {
    return Intl.message(
      'Can\'t get current location',
      name: 'cant_get_current_location',
      desc: '',
      args: [],
    );
  }

  /// `Please make sure you enable GPS and try again`
  String get pleaseMakeSureYouEnableGpsAndTryAgain {
    return Intl.message(
      'Please make sure you enable GPS and try again',
      name: 'please_make_sure_you_enable_gps_and_try_again',
      desc: '',
      args: [],
    );
  }

  /// `Search place`
  String get searchPlace {
    return Intl.message(
      'Search place',
      name: 'search_place',
      desc: '',
      args: [],
    );
  }

  /// `Finding place...`
  String get findingPlace {
    return Intl.message(
      'Finding place...',
      name: 'finding_place',
      desc: '',
      args: [],
    );
  }

  /// `No result found`
  String get noResultFound {
    return Intl.message(
      'No result found',
      name: 'no_result_found',
      desc: '',
      args: [],
    );
  }

  /// `Unnamed place`
  String get unnamedPlace {
    return Intl.message(
      'Unnamed place',
      name: 'unnamedPlace',
      desc: '',
      args: [],
    );
  }
}

class AppLocalizationDelegate extends LocalizationsDelegate<AppLocalizer> {
  const AppLocalizationDelegate();

  List<Locale> get supportedLocales {
    return const <Locale>[
      Locale.fromSubtags(languageCode: 'en'),
      Locale.fromSubtags(languageCode: 'ar'),
      Locale.fromSubtags(languageCode: 'de'),
      Locale.fromSubtags(languageCode: 'es'),
      Locale.fromSubtags(languageCode: 'fr'),
      Locale.fromSubtags(languageCode: 'it'),
      Locale.fromSubtags(languageCode: 'pt'),
      Locale.fromSubtags(languageCode: 'ru'),
      Locale.fromSubtags(languageCode: 'sr'),
      Locale.fromSubtags(languageCode: 'tr'),
    ];
  }

  @override
  bool isSupported(Locale locale) => _isSupported(locale);
  @override
  Future<AppLocalizer> load(Locale locale) => AppLocalizer.load(locale);
  @override
  bool shouldReload(AppLocalizationDelegate old) => false;

  bool _isSupported(Locale locale) {
    for (var supportedLocale in supportedLocales) {
      if (supportedLocale.languageCode == locale.languageCode) {
        return true;
      }
    }
    return false;
  }
}